# Generated by Django 3.1.2 on 2020-10-22 09:59

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0050_scene_baselayer"),
    ]

    operations = [
        migrations.AddField(
            model_name="layer",
            name="popup_config",
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
    ]



import math
from terra_layer.settings import (
    DEFAULT_CIRCLE_MIN_LEGEND_HEIGHT,
)

from .utils import (
    get_style_no_value_condition,
    get_positive_min_max,
    gen_style_interpolate,
    boundaries_round,
    circle_boundaries_candidate,
    circle_boundaries_filter_values,
)


def gen_proportionnal_radius_legend_items(
    min,
    max,
    max_value,
    color,
    no_value_size,
    no_value_color,
):
    """
    Generate a circle legend.
    """
    candidates = circle_boundaries_candidate(min, max)
    candidates = [max] + candidates + [min]
    boundaries = circle_boundaries_filter_values(
        candidates, max, max_value, DEFAULT_CIRCLE_MIN_LEGEND_HEIGHT
    )

    r = max_value / math.sqrt(max / math.pi)

    ret = [
        {
            "size": math.sqrt(b / math.pi) * r,
            "boundaries": {"lower": {"value": b}},
            "color": color,
        }
        for b in boundaries
    ]

    if no_value_size:
        ret.append(
            {
                "size": no_value_size * 2,
                "boundaries": {"lower": {"value": None}},
                "color": no_value_color,
            }
        )

    return ret


def gen_proportionnal_radius_style(geo_layer, data_field, map_field, prop_config):
    field_getter = ["get", data_field]
    max_value = prop_config["max_radius"]
    no_value = prop_config.get("no_value")

    # Get min max value
    mm = get_positive_min_max(geo_layer, data_field)

    if mm[1] is not None and mm[2] is not None:
        mm = boundaries_round(mm[1:])
        boundaries = [0, math.sqrt(mm[1] / math.pi)]
        sizes = [0, max_value / 2]

        radius_base = ["sqrt", ["/", field_getter, ["pi"]]]
        radius = gen_style_interpolate(radius_base, boundaries, sizes)

        return get_style_no_value_condition(
            field_getter,
            radius,
            no_value,
        )
    else:
        return no_value or 0


def gen_proportionnal_radius_legend(
    geo_layer, data_field, map_style_type, prop_config, color, no_value_color
):
    no_value_size = prop_config.get("no_value")
    max_value = prop_config["max_radius"]

    # Get min max value
    mm = get_positive_min_max(geo_layer, data_field)

    if mm[1] is not None and mm[2] is not None:
        mm = boundaries_round(mm[1:])

        return {
            "items": gen_proportionnal_radius_legend_items(
                mm[0],
                mm[1],
                max_value,
                color,
                no_value_size,
                no_value_color,
            ),
            "shape": "stackedCircle",
        }
    else:
        return {
            "items": [
                {
                    "size": no_value_size,
                    "color": no_value_color or color,
                    "boundaries": {
                        "lower": {"value": None, "included": True},
                        "upper": {"value": None, "included": True},
                    },
                }
            ],
            "shape": "stackedCircle",
        }



from django.db import connection
import numbers
import math
from functools import reduce

style_type_2_legend_shape = {
    "fill-extrusion": "square",
    "fill": "square",
    "circle": "circle",
    "symbol": "symbol",
    "line": "line",
}

stroke_color_props = ["line_color", "stroke_color", "outline_color"]
stroke_width_props = ["line_width", "stroke_width"]
shape_size_props = ["size", "radius"]


def style_type_2_legend_property(style_type):
    if any([s in style_type for s in stroke_color_props]):
        return "strokeColor"
    if any([s in style_type for s in stroke_width_props]):
        return "strokeWidth"
    if any([s in style_type for s in shape_size_props]):
        return "size"
    return "color"


def _flatten(levels):
    """
    Flatten 2-level array.
    [[1,2], [3, 4, 5]] -> [1, 2, 3, 4, 5]
    """
    return list(reduce(lambda x, y: x + y, levels or []))


def get_min_max(geo_layer, field):
    """
    Return the max and the min value of a property.
    """
    with connection.cursor() as cursor:
        cursor.execute(
            """
            SELECT
                bool_or((properties->>%(field)s)::numeric IS NULL) AS is_null,
                min((properties->>%(field)s)::numeric) AS min,
                max((properties->>%(field)s)::numeric) AS max
            FROM
                geostore_feature
            WHERE
                layer_id = %(layer_id)s
            """,
            {"field": field, "layer_id": geo_layer.id},
        )
        row = cursor.fetchone()
        is_null, min, max = row
        return [is_null == True, min, max]  # noqa


def get_positive_min_max(geo_layer, field):
    """
    Return the max and the min value of a property.
    """
    with connection.cursor() as cursor:
        cursor.execute(
            """
            SELECT
                bool_or((properties->>%(field)s)::numeric IS NULL) AS is_null,
                min((properties->>%(field)s)::numeric) AS min,
                max((properties->>%(field)s)::numeric) AS max
            FROM
                geostore_feature
            WHERE
                layer_id = %(layer_id)s AND
                (properties->>%(field)s)::numeric > 0
            """,
            {"field": field, "layer_id": geo_layer.id},
        )
        row = cursor.fetchone()
        is_null, min, max = row
        return [is_null == True, min, max]  # noqa


def discretize_quantile(geo_layer, field, class_count):
    """
    Compute Quantile class boundaries from a layer property.
    """
    with connection.cursor() as cursor:
        cursor.execute(
            """
            WITH
            ntiles AS (
                SELECT
                    (properties->>%(field)s)::numeric AS value,
                    ntile(%(class_count)s) OVER (ORDER BY (properties->>%(field)s)::numeric) AS ntile
                FROM
                    geostore_feature
                WHERE
                    layer_id = %(layer_id)s
                UNION ALL
                SELECT
                    NULL AS value,
                    NULL AS ntile
                FROM
                    geostore_feature
                WHERE
                    layer_id = %(layer_id)s AND
                    (properties->>%(field)s)::numeric IS NOT NULL
            )
            SELECT
                min(value) AS boundary,
                max(value) AS max
            FROM
                ntiles
            GROUP BY
                ntile
            ORDER BY
                ntile
            """,
            {"field": field, "class_count": class_count, "layer_id": geo_layer.id},
        )
        rows = cursor.fetchall()
        if rows and len(rows) > 0:
            rows = [row for row in rows if row[0] is not None]
            if len(rows) == 0:
                return []
            else:
                # Each class start + last class end
                return [r[0] for r in rows] + [rows[-1][1]]


def discretize_jenks(geo_layer, field, class_count):
    """
    Compute Jenks class boundaries from a layer property.
    Note: Use PostGIS ST_ClusterKMeans() as k-means function.
    """
    with connection.cursor() as cursor:
        cursor.execute(
            """
            WITH
            kmeans AS (
                SELECT
                    (properties->>%(field)s)::numeric AS field,
                    ST_ClusterKMeans(
                        ST_MakePoint((properties->>%(field)s)::numeric, 0),
                        least(%(class_count)s, (SELECT count(*) FROM geostore_feature WHERE layer_id = %(layer_id)s))::integer
                    ) OVER () AS class_id
                FROM
                    geostore_feature
                WHERE
                    layer_id = %(layer_id)s
            )
            SELECT
                min(field) AS min,
                max(field) AS max
            FROM
                kmeans
            GROUP BY
                class_id
            ORDER BY
                min,
                max
            """,
            {"field": field, "class_count": class_count, "layer_id": geo_layer.id},
        )
        rows = cursor.fetchall()
        if rows and len(rows) > 0:
            rows = [row for row in rows if row[0] is not None]
            if len(rows) == 0:
                return []
            else:
                # Each class start + last class end
                return [r[0] for r in rows] + [rows[-1][1]]


def discretize_equal_interval(geo_layer, field, class_count):
    """
    Compute QuantiEqual Interval class boundaries from a layer property.
    """
    is_null, min, max = get_min_max(geo_layer, field)
    if min is not None and max is not None and isinstance(min, numbers.Number):
        delta = (max - min) / class_count
        return [min + delta * i for i in range(0, class_count + 1)]
    else:
        return None


def discretize(geo_layer, field, method, class_count):
    """
    Select a method to compute class boundaries.
    Compute (len(class_count) + 1) boundaries.
    Note, can returns less boundaries than requested if lesser values in property than class_count
    """
    if method == "quantile":
        return discretize_quantile(geo_layer, field, class_count)
    elif method == "jenks":
        return discretize_jenks(geo_layer, field, class_count)
    elif method == "equal_interval":
        return discretize_equal_interval(geo_layer, field, class_count)
    else:
        raise ValueError(f'Unknow discretize method "{method}"')


def get_style_no_value_condition(key, with_value, with_no_value):
    if with_no_value is not None:
        if with_value is not None:
            return [
                "case",
                ["==", ["typeof", key], "number"],
                with_value,
                with_no_value,
            ]
        else:
            return with_no_value
    else:
        return with_value


def gen_style_steps(expression, boundaries, values):
    """
    Assume len(boundaries) <= len(colors) - 1
    """
    if len(boundaries) > 0:
        return ["step", expression, values[0]] + _flatten(
            zip(boundaries[1:], values[1:])
        )


def gen_style_interpolate(expression, boundaries, values):
    """
    Build a Mapbox GL Style interpolation expression.
    """
    return ["interpolate", ["linear"], expression] + _flatten(zip(boundaries, values))


def size_boundaries_candidate(min, max):
    return [(max - min) / 2]


def circle_boundaries_candidate(min, max):
    """
    Implementation of Self-Adjusting Legends for Proportional Symbol Maps
    https://pdfs.semanticscholar.org/d3f9/2bbd24ae83af6c101e5caacbd3e830d99272.pdf
    """
    if min is None or max is None:
        return []
    elif min <= 0:
        raise ValueError("Minimum value should be > 0")

    # array of base values in decreasing order in the range [1,10)
    bases = [5, 2.5, 1]
    # an index into the bases array
    base_id = 0
    # array that will hold the candidate values
    values = []
    # scale the minimum and maximum such that the minimum is > 1
    scale = 1  # scale factor
    min_s = min  # scaled minimum
    max_s = max  # scaled maximum
    while min_s < 1:
        min_s *= 10
        max_s *= 10
        scale /= 10

    # Compute the number of digits of the integral part of the scaled maximum
    ndigits = math.floor(math.log10(max_s))

    # Find the index into the bases array for the first limit smaller than max_s
    for i in range(0, len(bases)):
        if (max_s / 10 ** ndigits) >= bases[i]:
            base_id = i
            break

    while True:
        v = bases[base_id] * (10 ** ndigits)
        # stop the loop if the value is smaller than min_s
        if v <= min_s:
            break

        # otherwise store v in the values array
        values.append(
            v * scale
        )  # The the paper is false here, need mutiplication, no division
        # switch to the next base
        base_id += 1
        if base_id == len(bases):
            base_id = 0
            ndigits = ndigits - 1

    return values


def circle_boundaries_value_to_symbol_height(value, max_value, max_size):
    return math.sqrt(value / math.pi) * (max_size / math.sqrt(max_value / math.pi))


def circle_boundaries_filter_values(values, max_value, max_size, dmin):
    if not values or max_value is None or max_size is None:
        return []

    # array that will hold the filtered values
    filtered_values = []
    # add the maximum value
    filtered_values.append(values[0])
    # remember the height of the previously added value
    previous_height = circle_boundaries_value_to_symbol_height(
        values[0], max_value, max_size
    )
    # find the height and value of the smallest acceptable symbol
    last_height = 0
    last_value_id = len(values) - 1
    while last_value_id >= 0:
        last_height = circle_boundaries_value_to_symbol_height(
            values[last_value_id], max_value, max_size
        )
        if last_height > dmin:
            break
        last_value_id -= 1

    # loop over all values that are large enough
    for limit_id in range(1, last_value_id + 1):
        v = values[limit_id - 1]
        # compute the height of the symbol
        h = circle_boundaries_value_to_symbol_height(v, max_value, max_size)
        # do not draw the symbol if it is too close to the smallest symbol (but is not the smallest limit itself)
        if h - last_height < dmin and limit_id != last_value_id:
            continue
        # do not draw the symbol if it is too close to the previously drawn symbol
        if previous_height - h < dmin:
            continue
        filtered_values.append(v)
        # remember the height of the last drawn symbol
        previous_height = h

    return filtered_values


def lost_scale_digit(n, scale):
    """
    Return the number of digit to round
    """
    if n == 0:
        return 1  # Further in compuration 0 dived by 1 will be ok: 0
    else:
        return math.trunc(math.log10(n)) + 1 - scale


def trunc_scale(n, scale):
    lost = lost_scale_digit(n, scale)
    return math.trunc(n / (10 ** lost)) * (10 ** lost)


def round_scale(n, scale):
    lost = lost_scale_digit(n, scale)
    return round(n / (10 ** lost)) * (10 ** lost)


def ceil_scale(n, scale):
    lost = lost_scale_digit(n, scale)
    return math.ceil(n / (10 ** lost)) * (10 ** lost)


def boundaries_round(boundaries, scale=2):
    """
    Round boundaries to human readable number
    """
    return (
        [trunc_scale(boundaries[0], scale)]
        + [round_scale(b, scale) for b in boundaries[1:-2]]
        + [ceil_scale(boundaries[-1], scale)]
    )



from terra_layer.settings import (
    DEFAULT_NO_VALUE_FILL_COLOR,
)

from .utils import get_style_no_value_condition, style_type_2_legend_property

from .all import (
    gen_categorized_any_legend,
    gen_categorized_any_style,
)

from .color import (
    gen_graduated_color_legend,
    gen_graduated_color_style,
)

from .size import (
    gen_graduated_size_legend,
    gen_graduated_size_style,
    gen_proportionnal_size_legend,
    gen_proportionnal_size_style,
)

from .radius import (
    gen_proportionnal_radius_legend,
    gen_proportionnal_radius_style,
)


def to_map_style(prop):
    return prop.replace("_", "-")


def field_2_variation_type(field):
    if "color" in field:
        return "color"
    if (
        "width" in field
        or "height" in field
        or "opacity" in field
        or "intensity" in field
        or "size" in field
    ):
        return "value"
    if "radius" in field:
        return "radius"


def get_paint_or_layout(field):
    if field in [
        "icon-image",
        "icon-size",
        "text-field",
        "text-font",
        "text-size",
        "text-allow-overlap",
        "fill-sort-key",
        "line-sort-key",
    ]:
        return "layout"
    return "paint"


def get_layer_type(map_style_type):
    if map_style_type in ["icon", "text"]:
        return "symbol"
    return map_style_type


def generate_style_from_wizard(geo_layer, config):
    """
    Return a Mapbox GL Style and a Legend from a wizard setting.
    """

    # fill, fill_extrusion, line, text, symbol, circle
    map_style_type = config["map_style_type"]
    suid = config["uid"]

    map_style = {"type": get_layer_type(map_style_type)}

    if "min_zoom" in config:
        map_style["minzoom"] = config["min_zoom"]
    if "max_zoom" in config:
        map_style["maxzoom"] = config["max_zoom"]
    if "weight" in config:
        map_style["weight"] = config["weight"]

    legends = []

    for map_field, prop_config in config["style"].items():
        style_type = prop_config.get("type", "none")

        # Ignore style from other representation
        if not map_field.replace("fill_extrusion", "extrusion").startswith(
            map_style_type.replace("fill-extrusion", "extrusion")
        ):
            continue

        map_style_prop = to_map_style(map_field)
        paint_or_layout = get_paint_or_layout(map_style_prop)
        if style_type == "fixed":
            # Fixed value
            value = prop_config["value"]
            no_value = prop_config.get("no_value")
            data_field = prop_config.get("field")
            map_style.setdefault(paint_or_layout, {})[
                map_style_prop
            ] = get_style_no_value_condition(["get", data_field], value, no_value)
        elif style_type == "variable":
            # Variable style
            data_field = prop_config["field"]
            variation_type = field_2_variation_type(map_field)
            analysis = prop_config["analysis"]

            if variation_type == "color":
                if analysis == "graduated":
                    map_style.setdefault(paint_or_layout, {})[
                        map_style_prop
                    ] = gen_graduated_color_style(
                        geo_layer, data_field, map_field, prop_config
                    )
                    if prop_config.get("generate_legend"):
                        legend = gen_graduated_color_legend(
                            geo_layer,
                            data_field,
                            map_style_type,
                            prop_config,
                            style_type_2_legend_property(map_field),
                        )
                        legend["uid"] = f"{suid}__{map_field}"
                        # TODO reuse previous computations
                        legends.append(legend)
                elif analysis == "categorized":
                    map_style.setdefault(paint_or_layout, {})[
                        map_style_prop
                    ] = gen_categorized_any_style(
                        geo_layer, data_field, prop_config, DEFAULT_NO_VALUE_FILL_COLOR
                    )
                    if (
                        map_style.setdefault(paint_or_layout, {})[map_style_prop]
                        is None
                    ):
                        del map_style.setdefault(paint_or_layout, {})[map_style_prop]

                    if prop_config.get("generate_legend"):
                        legend = gen_categorized_any_legend(
                            map_style_type,
                            prop_config,
                            style_type_2_legend_property(map_field),
                        )
                        legend["uid"] = f"{suid}__{map_field}"
                        legends.append(legend)
                else:
                    raise ValueError(f'Unhandled analysis type "{analysis}"')

            if variation_type == "radius":
                if analysis == "categorized":
                    generated_style = gen_categorized_any_style(
                        geo_layer, data_field, prop_config, 0
                    )
                    if generated_style:
                        map_style.setdefault(paint_or_layout, {})[
                            map_style_prop
                        ] = generated_style
                    """if (
                        map_style[paint_or_layout][map_style_prop]
                        is None
                    ):
                        del map_style[paint_or_layout][map_style_prop]"""

                    if prop_config.get("generate_legend"):
                        color = (
                            config["style"]
                            .get(f"{map_style_type}_color", {})
                            .get("value", DEFAULT_NO_VALUE_FILL_COLOR)
                        )
                        legend = gen_categorized_any_legend(
                            map_style_type,
                            prop_config,
                            style_type_2_legend_property(map_field),
                            other_properties={"color": color},
                        )
                        legend["uid"] = f"{suid}__{map_field}"
                        legends.append(legend)

                elif analysis == "proportionnal":
                    map_style.setdefault(paint_or_layout, {})[
                        map_style_prop
                    ] = gen_proportionnal_radius_style(
                        geo_layer, data_field, map_field, prop_config
                    )
                    # Add sort key
                    # TODO find more smart way to do that
                    map_style["layout"] = {
                        f"{map_style_type}-sort-key": ["-", ["get", data_field]]
                    }
                    if prop_config.get("generate_legend"):
                        # TODO reuse previous computations
                        color = (
                            config["style"]
                            .get(f"{map_style_type}_color", {})
                            .get("value", DEFAULT_NO_VALUE_FILL_COLOR)
                        )
                        no_value_color = (
                            config["style"]
                            .get(f"{map_style_type}_color", {})
                            .get("no_value")
                        )
                        legend = gen_proportionnal_radius_legend(
                            geo_layer,
                            data_field,
                            map_style_type,
                            prop_config,
                            color,
                            no_value_color,
                        )
                        legend["uid"] = f"{suid}__{map_field}"
                        legends.append(legend)
                else:
                    raise ValueError(f'Unhandled analysis type "{analysis}"')

            if variation_type == "value":
                if analysis == "graduated":
                    map_style.setdefault(paint_or_layout, {})[
                        map_style_prop
                    ] = gen_graduated_size_style(
                        geo_layer, data_field, map_field, prop_config
                    )
                    if prop_config.get("generate_legend"):
                        # TODO reuse previous computations
                        color = (
                            config["style"]
                            .get(f"{map_style_type}_color", {})
                            .get("value", DEFAULT_NO_VALUE_FILL_COLOR)
                        )
                        no_value_color = (
                            config["style"]
                            .get(f"{map_style_type}_color", {})
                            .get("no_value")
                        )
                        legend = gen_graduated_size_legend(
                            geo_layer,
                            data_field,
                            map_style_type,
                            prop_config,
                            color,
                            no_value_color,
                            style_type_2_legend_property(map_field),
                        )
                        legend["uid"] = f"{suid}__{map_field}"
                        legends.append(legend)

                elif analysis == "categorized":
                    generated_style = gen_categorized_any_style(
                        geo_layer, data_field, prop_config, 0
                    )
                    if generated_style:
                        map_style.setdefault(paint_or_layout, {})[
                            map_style_prop
                        ] = generated_style

                    if prop_config.get("generate_legend"):
                        color = (
                            config["style"]
                            .get(f"{map_style_type}_color", {})
                            .get("value", DEFAULT_NO_VALUE_FILL_COLOR)
                        )
                        legend = gen_categorized_any_legend(
                            map_style_type,
                            prop_config,
                            style_type_2_legend_property(map_field),
                            other_properties={"color": color},
                        )
                        legend["uid"] = f"{suid}__{map_field}"
                        legends.append(legend)

                elif analysis == "proportionnal":
                    """map_style["layout"] = {
                        f"{map_style_type}-sort-key": ["-", ["get", data_field]]
                    }"""
                    map_style.setdefault(paint_or_layout, {})[
                        map_style_prop
                    ] = gen_proportionnal_size_style(
                        geo_layer, data_field, map_field, prop_config
                    )
                    if prop_config.get("generate_legend"):
                        # TODO reuse previous computations
                        color = (
                            config["style"]
                            .get(f"{map_style_type}_color", {})
                            .get("value", DEFAULT_NO_VALUE_FILL_COLOR)
                        )
                        no_value_color = (
                            config["style"]
                            .get(f"{map_style_type}_color", {})
                            .get("no_value")
                        )
                        legend = gen_proportionnal_size_legend(
                            geo_layer,
                            data_field,
                            map_style_type,
                            prop_config,
                            color,
                            no_value_color,
                            style_type_2_legend_property(map_field),
                        )
                        legend["uid"] = f"{suid}__{map_field}"
                        legends.append(legend)
                else:
                    raise ValueError(f'Unknow analysis type "{analysis}"')

    return (map_style, legends)



# Generated by Django 2.2.17 on 2020-11-30 08:28

from django.db import migrations, transaction


def fill_style_name(field):
    if field == "stroke_color":
        return "fill_outline_color"
    else:
        return field


def circle_style_name(field):
    if field == "stroke_color":
        return "circle_stroke_color"
    elif field == "stroke_width":
        return "circle_stroke_width"
    elif field == "fill_opacity":
        return "circle_opacity"
    else:
        return field


def migrate_graduated_wizard(layer):
    field = layer.layer_style_wizard["field"]
    method = layer.layer_style_wizard.get("method")
    boundaries = layer.layer_style_wizard.get("boundaries")
    fill_colors = layer.layer_style_wizard.get("style", layer.layer_style_wizard)[
        "fill_color"
    ]

    # Other fields
    fields = {}
    no_value_fields = {}

    for f in ["fill_color", "fill_opacity", "stroke_color"]:
        fields[f] = layer.layer_style_wizard.get("style", layer.layer_style_wizard).get(
            f
        )
        no_value_fields[f] = layer.layer_style_wizard.get("no_value_style", {}).get(f)

    layer.main_style = {
        "type": "wizard",
        "map_style_type": "fill",
        "style": {
            "fill_color": {
                "type": "variable",
                "field": field,
                "values": fill_colors,
            },
        },
    }
    if boundaries:
        layer.main_style["style"]["fill_color"]["boundaries"] = boundaries
    if method:
        layer.main_style["style"]["fill_color"]["method"] = method
    if no_value_fields["fill_color"]:
        layer.main_style["style"]["fill_color"]["no_value"] = no_value_fields[
            "fill_color"
        ]

    del fields["fill_color"]

    for field, value in fields.items():
        if value:
            layer.main_style["style"][fill_style_name(field)] = {
                "type": "fixed",
                "value": value,
            }
            if fields[field]:
                layer.main_style["style"][fill_style_name(field)]["no_value"] = fields[
                    field
                ]


def migrate_circle_wizard(layer):
    field = layer.layer_style_wizard["field"]
    max_diameter = layer.layer_style_wizard["max_diameter"]
    fields = {}
    no_value_fields = {}

    for f in [
        "circle_radius",
        "fill_color",
        "fill_opacity",
        "stroke_color",
        "stroke_width",
    ]:
        fields[f] = layer.layer_style_wizard["style"].get(f)
        no_value_fields[f] = layer.layer_style_wizard.get("no_value_style", {}).get(f)

    layer.main_style = {
        "type": "wizard",
        "map_style_type": "circle",
        "style": {
            "circle_radius": {
                "type": "variable",
                "field": field,
                "max_radius": max_diameter,
            },
        },
    }

    if no_value_fields["circle_radius"]:
        layer.main_style["style"]["circle_radius"]["no_value"] = no_value_fields[
            "circle_radius"
        ]

    del fields["circle_radius"]

    for field, value in fields.items():
        if value:
            layer.main_style["style"][circle_style_name(field)] = {
                "type": "fixed",
                "value": value,
            }
            if fields[field]:
                layer.main_style["style"][circle_style_name(field)][
                    "no_value"
                ] = fields[field]


@transaction.atomic
def forward(apps, schema_editor):
    Layer = apps.get_model("terra_layer", "Layer")

    for layer in Layer.objects.all():
        if layer.main_style:
            print(f"Nothing to do for {layer}. Already migrated ?")
            continue

        if layer.layer_style_wizard:
            print(f"Wizard style for {layer}")
            if layer.layer_style_wizard["symbology"] == "graduated":
                migrate_graduated_wizard(layer)
            elif layer.layer_style_wizard["symbology"] == "circle":
                migrate_graduated_wizard(layer)
        elif layer.layer_style:
            print(f"Advanced style for {layer}")
            layer.main_style = {
                "type": "advanced",
                "map_style_type": layer.layer_style["type"],
            }
        else:
            print(f"No style found for layer {layer}")

        # Keep previously generated style
        layer.main_style["map_style"] = layer.layer_style
        layer.save()

        # Migrate extra styles
        for extra in layer.extra_styles.all():
            if extra.style:
                extra.style_config = {
                    "type": "advanced",
                    "map_style_type": extra.style.get(
                        "type",
                        list(extra.style.get("paint", {"fill-color": ""}).keys())[
                            0
                        ].split("-")[0],
                    ),
                    "map_style": extra.style,
                }
                extra.save()


def backward(apps, schema_editor):
    Layer = apps.get_model("terra_layer", "Layer")

    for layer in Layer.objects.all():
        layer.layer_style = layer.main_style["map_style"]
        # Here we loose the wizard. Too hard to migrate.
        layer.save()


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0057_add_new_style_field"),
    ]

    operations = [
        migrations.RunPython(forward, reverse_code=backward),
    ]
